/*
 * Copyright (c) 2019 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.google.api.services.samples.adsdatahub.cmdline;

import com.google.api.services.adsdatahub.v1.AdsDataHub;
import com.google.api.services.adsdatahub.v1.model.AddEventMatchersRequest;
import com.google.api.services.adsdatahub.v1.model.EventMatcher;
import com.google.api.services.adsdatahub.v1.model.FrequencyBasedListInfo;
import com.google.api.services.adsdatahub.v1.model.UserList;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * This sample illustrates how to add event matchers to a frequency list.
 *
 * <p>See the <a
 * href="https://developers.google.com/ads-data-hub/reference/rest/v1/customers.userLists/addEventMatchers">
 * customers.userLists/addEventMatchers reference documentation</a> for more details.
 */
public class AddEventMatchersToFrequencyList extends BaseSample {

  @Override
  public String getName() {
    return "Add event matchers";
  }

  @Override
  public String getDescription() {
    return "Adds event matchers (defining which events to count) to an existing frequency list.";
  }

  @Override
  public void execute(AdsDataHub client) throws IOException {
    String name =
        getStringInput(
            "name",
            "Enter the fully qualified resource name of the existing"
                + " user list (Format: \"customers/[customer_id]/userLists/[user_list_id]\")");
    String matchType =
        getStringInput("matchType", "Enter the match type (example: GOOGLE_ADS_CAMPAIGN_ID)");
    List<String> values =
        getStringListInput(
            "matchValues", "Enter a comma separated list of values for the given match type");

    ArrayList matchers = new ArrayList<EventMatcher>();
    for (String value : values) {
      matchers.add(new EventMatcher().setMatchType(matchType).setValue(value));
    }

    UserList updatedUserList =
        client
            .customers()
            .userLists()
            .addEventMatchers(name, new AddEventMatchersRequest().setEventMatchers(matchers))
            .execute();

    System.out.println("========================================");
    System.out.printf("Updated frequency based list: \"%s\"%n", updatedUserList.getName());
    System.out.println("========================================");
    System.out.printf("\tUser list ID: %s\n", updatedUserList.getUserListId());
    System.out.printf("\tUser list title: %s\n", updatedUserList.getTitle());
    System.out.printf("\tAds data customer ID: %s\n", updatedUserList.getAdsDataCustomerId());
    System.out.printf(
        "\tRecipient Ads Data Links: %s\n", updatedUserList.getRecipientAdsDataLinks());
    FrequencyBasedListInfo frequencyList = updatedUserList.getFrequencyList();
    System.out.printf(
        "\tFrequency sliding window (in days): %s\n", frequencyList.getSlidingWindowDays());
    System.out.printf("\tDV360 Line Item IDs: %s\n", frequencyList.getDv360LineitemIds());
    System.out.printf("\tCM Campaign IDs: %s\n", frequencyList.getCmCampaignIds());
    System.out.printf("\tGoogleAds Campaign IDs: %s\n", frequencyList.getGoogleAdsCampaignIds());
    System.out.printf("\tEvent Matchers: %s\n", frequencyList.getEventMatchers());
  }
}
