/*
 * Copyright (c) 2019 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.google.api.services.samples.adsdatahub.cmdline;

import com.google.api.client.auth.oauth2.Credential;
import com.google.api.client.extensions.java6.auth.oauth2.AuthorizationCodeInstalledApp;
import com.google.api.client.extensions.jetty.auth.oauth2.LocalServerReceiver;
import com.google.api.client.googleapis.auth.oauth2.GoogleAuthorizationCodeFlow;
import com.google.api.client.googleapis.auth.oauth2.GoogleClientSecrets;
import com.google.api.client.googleapis.javanet.GoogleNetHttpTransport;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.jackson2.JacksonFactory;
import com.google.api.client.util.store.FileDataStoreFactory;
import com.google.api.services.adsdatahub.v1.AdsDataHub;
import com.google.common.collect.ImmutableList;
import com.google.common.io.Files;
import java.io.IOException;
import java.io.InputStreamReader;
import java.security.GeneralSecurityException;
import java.util.ArrayList;

/**
 * A sample application that runs multiple requests against the Ads Data Hub v1 API. These include:
 *
 * <ul>
 *   <li>List Customers
 *   <li>List Ads Data Links
 *   <li>Create Query
 *   <li>List Queries
 *   <li>Start Query
 *   <li>Create User List
 *   <li>Create Frequency Based User List
 *   <li>List User Lists
 *   <li>Add/Remove Recipients to/from User List
 *   <li>Describe Valid Event Matchers
 *   <li>Add Event Matchers to Frequency Based User List
 *   <li>Create User List Query
 *   <li>Update User List Query
 *   <li>List User List Queries
 *   <li>Start User List Query
 *   <li>Generate Frequency List Memberships
 * </ul>
 */
public class AdsDataHubSample {
  /**
   * Be sure to specify the name of your application. If the application name is {@code null} or
   * blank, the application will log a warning. Suggested format is "MyCompany-ProductName/1.0".
   */
  private static final String APPLICATION_NAME = System.getProperty("sample.name", "SampleCode/1.1");

  /** Full path to the client secrets JSON file downloaded from the Google Developer's Console. */
  private static final java.io.File CLIENT_SECRETS_FILE =
      new java.io.File(System.getProperty("sample.secrets", "INSERT_PATH_TO_CLIENT_SECRETS_FILE"));

  /** Path to directory used to store credentials generated from client secrets. */
  private static final java.io.File CREDENTIALS_DIRECTORY =
      new java.io.File(System.getProperty("sample.credentials", "adh-credentials"));

  /** Global instance of the File DataStore factory. */
  private static FileDataStoreFactory dataStoreFactory;

  /** Global instance of the HTTP transport. */
  private static HttpTransport httpTransport;

  /** Global instance of the JSON factory. */
  private static final JsonFactory JSON_FACTORY = JacksonFactory.getDefaultInstance();

  private static ArrayList<BaseSample> samples;

  /**
   * Authorizes the installed application to access user's protected data.
   *
   * @throws IOException
   * @throws GeneralSecurityException
   */
  private static Credential authorize() throws Exception {
    httpTransport = GoogleNetHttpTransport.newTrustedTransport();
    dataStoreFactory = new FileDataStoreFactory(CREDENTIALS_DIRECTORY);

    GoogleClientSecrets clientSecrets =
        GoogleClientSecrets.load(
            JSON_FACTORY,
            new InputStreamReader(Files.asByteSource(CLIENT_SECRETS_FILE).openStream()));

    GoogleAuthorizationCodeFlow flow =
        new GoogleAuthorizationCodeFlow.Builder(
                httpTransport,
                JSON_FACTORY,
                clientSecrets,
                ImmutableList.of("https://www.googleapis.com/auth/adsdatahub"))
            .setDataStoreFactory(dataStoreFactory)
            .build();

    return new AuthorizationCodeInstalledApp(flow, new LocalServerReceiver()).authorize("user");
  }

  /**
   * Performs all necessary setup steps for running requests against the AdsDataHub v1 API.
   *
   * @return An initialized AdsDataHub service object.
   */
  private static AdsDataHub initClient(Credential credential) {
    AdsDataHub client =
        new AdsDataHub.Builder(httpTransport, JSON_FACTORY, credential)
            .setApplicationName(APPLICATION_NAME)
            .build();
    return client;
  }

  /** Initializes the list of available code samples. */
  private static void initSamples() {
    samples = new ArrayList<>();
    samples.add(new ListCustomers());
    samples.add(new CreateQuery());
    samples.add(new ListQueries());
    samples.add(new StartQuery());
    samples.add(new CreateUserList());
    samples.add(new ListUserLists());
    samples.add(new ListAdsDataLinks());
    samples.add(new AddRecipientsToUserList());
    samples.add(new RemoveRecipientsFromUserList());
    samples.add(new CreateUserListQuery());
    samples.add(new ListUserListQueries());
    samples.add(new UpdateUserListQuery());
    samples.add(new StartUserListQuery());
    samples.add(new DescribeValidFrequencyLimitingEventMatchers());
    samples.add(new CreateFrequencyBasedUserList());
    samples.add(new AddEventMatchersToFrequencyList());
    samples.add(new GenerateFrequencyListMemberships());
  }

  /**
   * Runs all the Ads Data Hub v1 REST API samples.
   *
   * @param args command-line arguments.
   */
  public static void main(String[] args) throws Exception {
    httpTransport = GoogleNetHttpTransport.newTrustedTransport();
    initSamples();
    Credential credentials = authorize();
    AdsDataHub client = initClient(credentials);
    BaseSample sample = null;

    while ((sample = selectSample()) != null) {
      try {
        System.out.printf("%nExecuting sample: %s%n%n", sample.getName());
        sample.execute(client);
      } catch (IOException e) {
        e.printStackTrace();
      }
    }
  }

  /**
   * Prints the list of available code samples and prompts the user to select one.
   *
   * @return The selected sample or null if the user selected to exit.
   */
  private static BaseSample selectSample() throws IOException {
    System.out.printf("Samples:%n");
    int counter = 1;
    for (BaseSample sample : samples) {
      System.out.printf("%d) %s - %s%n", counter++, sample.getName(), sample.getDescription());
    }
    System.out.printf("%d) Exit the program%n", counter++);
    Integer sampleNumber = null;
    while (sampleNumber == null) {
      try {
        System.out.println("Select a sample number and press enter:");
        sampleNumber = Integer.parseInt(Utils.readInputLine());
        if (sampleNumber < 1 || sampleNumber > samples.size()) {
          if (sampleNumber == samples.size() + 1) {
            return null;
          }
          System.out.printf("Invalid number provided, try again%n");
          sampleNumber = null;
        }
      } catch (NumberFormatException e) {
        System.out.printf("Invalid number provided, try again%n");
      }
    }
    return samples.get(sampleNumber - 1);
  }
}
