/*
 * Copyright (c) 2019 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.google.api.services.samples.adsdatahub.cmdline;

import com.google.api.services.adsdatahub.v1.AdsDataHub;
import com.google.api.services.adsdatahub.v1.AdsDataHub.Customers.UserLists.Create;
import com.google.api.services.adsdatahub.v1.model.FrequencyBasedListInfo;
import com.google.api.services.adsdatahub.v1.model.FrequencyConstraint;
import com.google.api.services.adsdatahub.v1.model.UserList;
import java.io.IOException;
import java.util.List;

/**
 * This sample illustrates how to create a frequency based user list.
 *
 * <p>See the <a
 * href="https://developers.google.com/ads-data-hub/reference/rest/v1/customers.userLists/create">
 * customers.userLists/create reference documentation</a> for more details.
 */
public class CreateFrequencyBasedUserList extends BaseSample {

  @Override
  public String getName() {
    return "Create frequency based user list";
  }

  @Override
  public String getDescription() {
    return "Creates a frequency based user list for the specified customer.";
  }

  @Override
  public void execute(AdsDataHub client) throws IOException {
    Long customerId = getLongInput("customerId", "Enter the managing customer ID");
    Long adsDataCustomerId =
        getLongInput("adsDataCustomerId", "Enter the ad data owning customer ID");
    String title = getStringInput("title", "Enter a unique title for the user list");
    List<Long> dv360LineItemIds =
        getOptionalLongListInput(
            "dv360LineItemIds", "Enter a comma separated list of DV360 Line Item IDs");
    List<Long> cmCampaignIds =
        getOptionalLongListInput(
            "cmCampaignIds", "Enter a comma separated list of CM Campaign IDs");
    List<Long> googleAdsCampaignIds =
        getOptionalLongListInput(
            "googleAdsCampaignIds", "Enter a comma separated list of GoogleAds Campaign IDs");
    Integer slidingWindowDays =
        getIntInput("slidingWindowDays", "Enter the window length in days for frequency detection");
    Long threshold =
        getLongInput(
            "threshold", "Enter the number of impressions to cause a user to be added to the list");
    List<String> recipients =
        getOptionalStringListInput(
            "recipientAdsDataLinks",
            "Enter a comma separated list of fully qualified resource names for the AdsDataLinks"
                + " that represent recipients that the user list should be shared with (Format:"
                + " \"customers/[customer_id]/adsDataLinks/[resource_id]\")");

    UserList userList =
        new UserList()
            .setAdsDataCustomerId(adsDataCustomerId)
            .setTitle(title)
            .setRecipientAdsDataLinks(recipients)
            .setFrequencyList(
                new FrequencyBasedListInfo()
                    .setDv360LineitemIds(dv360LineItemIds)
                    .setCmCampaignIds(cmCampaignIds)
                    .setGoogleAdsCampaignIds(googleAdsCampaignIds)
                    .setSlidingWindowDays(slidingWindowDays)
                    .setFrequencyConstraint(
                        new FrequencyConstraint()
                            .setEventType("IMPRESSIONS")
                            .setThresholdEventCount(threshold)));

    Create request = client.customers().userLists().create("customers/" + customerId, userList);
    UserList createdUserList = request.execute();

    System.out.println("========================================");
    System.out.printf("Frequency based list created for customer ID: \"%s\"%n", customerId);
    System.out.println("========================================");
    System.out.printf("Name: %s\n", createdUserList.getName());
    System.out.printf("\tUser list ID: %s\n", createdUserList.getUserListId());
    System.out.printf("\tUser list title: %s\n", createdUserList.getTitle());
    System.out.printf("\tAds data customer ID: %s\n", createdUserList.getAdsDataCustomerId());
    System.out.printf(
        "\tRecipient Ads Data Links: %s\n", createdUserList.getRecipientAdsDataLinks());
    FrequencyBasedListInfo frequencyList = createdUserList.getFrequencyList();
    System.out.printf(
        "\tFrequency sliding window (in days): %s\n", frequencyList.getSlidingWindowDays());
    System.out.printf("\tDV360 Line Item IDs: %s\n", frequencyList.getDv360LineitemIds());
    System.out.printf("\tCM Campaign IDs: %s\n", frequencyList.getCmCampaignIds());
    System.out.printf("\tGoogleAds Campaign IDs: %s\n", frequencyList.getGoogleAdsCampaignIds());
    System.out.printf("\tEvent Matchers: %s\n", frequencyList.getEventMatchers());
  }
}
