/*
 * Copyright (c) 2019 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.google.api.services.samples.adsdatahub.cmdline;

import com.google.api.services.adsdatahub.v1.AdsDataHub;
import com.google.api.services.adsdatahub.v1.AdsDataHub.Customers.AnalysisQueries.Create;
import com.google.api.services.adsdatahub.v1.model.AnalysisQuery;
import com.google.api.services.adsdatahub.v1.model.FieldType;
import com.google.api.services.adsdatahub.v1.model.ParameterType;
import com.google.common.collect.ImmutableMap;
import java.io.IOException;
import java.util.Map;

/**
 * This sample illustrates how to create a query for the specified customer.
 *
 * <p>See the <a href="customers.analysisQueries.create reference
 * documentation">https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries/create</a>
 * for more details.
 */
public class CreateQuery extends BaseSample {

  private static String QUERY_TEXT =
      "WITH stats AS (\n"
          + "  SELECT\n"
          + "    campaign_id,\n"
          + "    demographics.gender AS gender_id,\n"
          + "    demographics.age_group AS age_group_id,\n"
          + "    COUNT(DISTINCT user_id) AS users,\n"
          + "    COUNT(*) AS impressions\n"
          + "  FROM adh.google_ads_impressions\n"
          + "  WHERE customer_id = @customer_id\n"
          + "  GROUP BY 1, 2, 3\n"
          + ")\n"
          + "SELECT\n"
          + "  campaign_name,\n"
          + "  gender_name,\n"
          + "  age_group_name,\n"
          + "  users,\n"
          + "  impressions\n"
          + "FROM stats\n"
          + "  LEFT JOIN adh.google_ads_campaign USING (campaign_id)\n"
          + "  LEFT JOIN adh.gender USING (gender_id)\n"
          + "  LEFT JOIN adh.age_group USING (age_group_id)\n"
          + "ORDER BY 1, 2, 3";

  private static Map<String, ParameterType> PARAM_TYPES =
      ImmutableMap.of("customer_id", new ParameterType().setType(new FieldType().setType("INT64")));

  @Override
  public String getName() {
    return "Create query";
  }

  @Override
  public String getDescription() {
    return "Creates query for the specified customer.";
  }

  @Override
  public void execute(AdsDataHub client) throws IOException {
    Long customerId = getLongInput("customerId", "Enter the customer ID");
    String title = getStringInput("title", "Enter a unique title for the query");

    AnalysisQuery query = new AnalysisQuery();
    query.setTitle(title);
    query.setQueryText(QUERY_TEXT);
    query.setParameterTypes(PARAM_TYPES);

    Create request = client.customers().analysisQueries().create("customers/" + customerId, query);
    AnalysisQuery createdQuery = request.execute();

    System.out.println("========================================");
    System.out.printf("Query created for customer ID: \"%s\"%n", customerId);
    System.out.println("========================================");
    System.out.printf("Name: %s\n", createdQuery.getName());
    System.out.printf("\tQuery state: %s\n", createdQuery.getQueryState());
    System.out.printf("\tQuery text: \n\"%s\"\n", createdQuery.getQueryText());
    System.out.printf("\tParameter types: %s\n\n", createdQuery.getParameterTypes());
  }
}
