/*
 * Copyright (c) 2019 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.google.api.services.samples.adsdatahub.cmdline;

import com.google.api.services.adsdatahub.v1.AdsDataHub;
import com.google.api.services.adsdatahub.v1.AdsDataHub.Customers.UserListQueries.Create;
import com.google.api.services.adsdatahub.v1.model.FieldType;
import com.google.api.services.adsdatahub.v1.model.ParameterType;
import com.google.api.services.adsdatahub.v1.model.UserListQuery;
import com.google.common.collect.ImmutableMap;
import java.io.IOException;
import java.util.Map;

/**
 * This sample illustrates how to create a user list query for the specified customer.
 *
 * <p>See the <a href="customers.userListQueries.create reference
 * documentation">https://developers.google.com/ads-data-hub/reference/rest/v1/customers.userListQueries/create</a>
 * for more details.
 */
public class CreateUserListQuery extends BaseSample {

  private static String QUERY_TEXT =
      "SELECT\n"
          + "  user_id\n"
          + "FROM adh.google_ads_clicks\n"
          + "WHERE campaign_id = @campaign_id";

  private static Map<String, ParameterType> PARAM_TYPES =
      ImmutableMap.of("campaign_id", new ParameterType().setType(new FieldType().setType("INT64")));

  @Override
  public String getName() {
    return "Create user list query";
  }

  @Override
  public String getDescription() {
    return "Creates a user list query for the specified customer.";
  }

  @Override
  public void execute(AdsDataHub client) throws IOException {
    Long customerId = getLongInput("customerId", "Enter the customer ID");
    String title = getStringInput("title", "Enter a unique title for the query");

    UserListQuery query = new UserListQuery();
    query.setTitle(title);
    query.setQueryText(QUERY_TEXT);
    query.setParameterTypes(PARAM_TYPES);

    Create request = client.customers().userListQueries().create("customers/" + customerId, query);
    UserListQuery createdQuery = request.execute();

    System.out.println("========================================");
    System.out.printf("User list query created for customer ID: \"%s\"%n", customerId);
    System.out.println("========================================");
    System.out.printf("Name: %s\n", createdQuery.getName());
    System.out.printf("\tQuery text: \n\"%s\"\n", createdQuery.getQueryText());
    System.out.printf("\tParameter types: %s\n\n", createdQuery.getParameterTypes());
  }
}
