/*
 * Copyright (c) 2019 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.google.api.services.samples.adsdatahub.cmdline;

import com.google.api.services.adsdatahub.v1.AdsDataHub;
import com.google.api.services.adsdatahub.v1.model.DescribeValidFrequencyLimitingEventMatchersResponse;
import com.google.api.services.adsdatahub.v1.model.EventMatcherInfo;
import java.io.IOException;
import java.util.List;

/**
 * This sample illustrates how to retrieve valid event matchers for Frequency Based Lists.
 *
 * <p>See the <a
 * href="https://developers.google.com/ads-data-hub/reference/rest/v1/customers/describeValidFrequencyLimitingEventMatchers">
 * customers/describeValidFrequencyLimitingEventMatchers reference documentation</a> for more
 * details.
 */
public class DescribeValidFrequencyLimitingEventMatchers extends BaseSample {

  @Override
  public String getName() {
    return "Describe Frequency Limiting event matchers";
  }

  @Override
  public String getDescription() {
    return "Lists out valid frequency limiting event matchers for a specified data customer.";
  }

  @Override
  public void execute(AdsDataHub client) throws IOException {
    Long customerId = getLongInput("customerId", "Enter the customer ID");
    Long adsDataCustomerId = getLongInput("adsDataCustomerId", "Enter the ads data customer ID");
    String filter =
        getOptionalStringInput(
            "eventMatchersFilter",
            "Enter filter (example: event_matcher.match_type = DV360_LINE_ITEM_ID)");
    String pageToken = null;
    boolean noData = true;

    System.out.println("========================================");
    System.out.printf(
        "Listing of frequency limiting event matchers for customer: %s and ads data customer: %s\n",
        customerId, adsDataCustomerId);
    System.out.println("========================================");

    do {
      DescribeValidFrequencyLimitingEventMatchersResponse response =
          client
              .customers()
              .describeValidFrequencyLimitingEventMatchers("customers/" + customerId)
              .setAdsDataCustomerId(adsDataCustomerId)
              .setPageToken(pageToken)
              .execute();
      pageToken = response.getNextPageToken();
      List<EventMatcherInfo> eventMatcherInfos = response.getEventMatcherInfos();

      if (eventMatcherInfos != null && eventMatcherInfos.size() > 0) {
        noData = false;
        for (EventMatcherInfo eventMatcherInfo : eventMatcherInfos) {
          System.out.printf(
              "* Display Name: %s\n", eventMatcherInfo.getEventMatcherMetadata().getDisplayName());
          System.out.printf("\tEvent Matcher: %s\n", eventMatcherInfo.getEventMatcher());
          System.out.printf("\tMetadata: %s\n", eventMatcherInfo.getEventMatcherMetadata());
        }
      }
    } while (pageToken != null);

    if (noData) {
      System.out.println("No valid event matchers were found.");
    }
  }
}
