/*
 * Copyright (c) 2019 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.google.api.services.samples.adsdatahub.cmdline;

import com.google.api.services.adsdatahub.v1.AdsDataHub;
import com.google.api.services.adsdatahub.v1.model.AnalysisQuery;
import com.google.api.services.adsdatahub.v1.model.ListAnalysisQueriesResponse;
import java.io.IOException;
import java.util.List;

/**
 * This sample illustrates how to retrieve all queries for the specified customer.
 *
 * <p>See the <a href="customers.analysisQueries.list reference
 * documentation">https://developers.google.com/ads-data-hub/reference/rest/v1/customers.analysisQueries/list</a>
 * for more details.
 */
public class ListQueries extends BaseSample {

  @Override
  public String getName() {
    return "List queries";
  }

  @Override
  public String getDescription() {
    return "Lists queries for the specified customer.";
  }

  @Override
  public void execute(AdsDataHub client) throws IOException {
    Long customerId = getLongInput("customerId", "Enter the customer ID");
    String pageToken = null;
    boolean noData = true;

    System.out.println("========================================");
    System.out.printf("Listing of analysis queries associated with customerId: %s\n", customerId);
    System.out.println("========================================");

    do {
      ListAnalysisQueriesResponse response =
          client
              .customers()
              .analysisQueries()
              .list("customers/" + customerId)
              .setPageToken(pageToken)
              .execute();
      pageToken = response.getNextPageToken();
      List<AnalysisQuery> queries = response.getQueries();

      if (queries != null && queries.size() > 0) {
        noData = false;
        for (AnalysisQuery query : queries) {
          System.out.printf("* Title: %s\n", query.getTitle());
          System.out.printf("\tQuery name: %s\n", query.getName());
          System.out.printf("\tQuery state: %s\n", query.getQueryState());
          System.out.printf("\tQuery text: \n\"%s\"\n", query.getQueryText());
          System.out.printf("\tParameter types: %s\n\n", query.getParameterTypes());
        }
      }
    } while (pageToken != null);

    if (noData) {
      System.out.println("No queries were found.");
    }
  }
}
