/*
 * Copyright (c) 2019 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.google.api.services.samples.adsdatahub.cmdline;

import com.google.api.services.adsdatahub.v1.AdsDataHub;
import com.google.api.services.adsdatahub.v1.model.ListUserListsResponse;
import com.google.api.services.adsdatahub.v1.model.UserList;
import java.io.IOException;
import java.util.List;

/**
 * This sample illustrates how to retrieve all user lists managed by the specified customer.
 *
 * <p>See the <a
 * href="https://developers.google.com/ads-data-hub/reference/rest/v1/customers.userLists/list">
 * customers.userLists/list reference documentation</a> for more details.
 */
public class ListUserLists extends BaseSample {

  @Override
  public String getName() {
    return "List user lists";
  }

  @Override
  public String getDescription() {
    return "Lists user lists managed by the specified customer.";
  }

  @Override
  public void execute(AdsDataHub client) throws IOException {
    Long customerId = getLongInput("customerId", "Enter the customer ID");
    String filter =
        getOptionalStringInput(
            "userListFilter", "Enter a filter (example: type = FREQUENCY_BASED)");
    String pageToken = null;
    boolean noData = true;

    System.out.println("========================================");
    System.out.printf("Listing of user lists managed by customerId: %s\n", customerId);
    System.out.println("========================================");

    do {
      ListUserListsResponse response =
          client
              .customers()
              .userLists()
              .list("customers/" + customerId)
              .setFilter(filter)
              .setPageToken(pageToken)
              .execute();
      pageToken = response.getNextPageToken();
      List<UserList> userLists = response.getUserLists();

      if (userLists != null && userLists.size() > 0) {
        noData = false;
        for (UserList userList : userLists) {
          System.out.printf("* Title: %s\n", userList.getTitle());
          System.out.printf("\tUser list id: %s\n", userList.getUserListId());
          System.out.printf("\tUser list name: %s\n", userList.getName());
          System.out.printf("\tUser list description: %s\n", userList.getDescription());
          System.out.printf("\tAds data customer: %s\n", userList.getAdsDataCustomerId());
          System.out.printf(
              "\tRecipient Ads Data Links: %s\n", userList.getRecipientAdsDataLinks());
        }
      }
    } while (pageToken != null);

    if (noData) {
      System.out.println("No user lists were found.");
    }
  }
}
