/*
 * Copyright (c) 2019 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.google.api.services.samples.adsdatahub.cmdline;

import com.google.api.services.adsdatahub.v1.AdsDataHub;
import com.google.api.services.adsdatahub.v1.model.Date;
import com.google.api.services.adsdatahub.v1.model.Operation;
import com.google.api.services.adsdatahub.v1.model.ParameterValue;
import com.google.api.services.adsdatahub.v1.model.QueryExecutionSpec;
import com.google.api.services.adsdatahub.v1.model.StartUserListQueryRequest;
import com.google.api.services.adsdatahub.v1.model.Status;
import com.google.common.collect.ImmutableMap;
import java.io.IOException;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

/**
 * This sample illustrates how to start a user list query with the specified name.
 *
 * <p>In order to run a named user list query, you must first create a query and user list. The
 * adsDataCustomerId from the user list must match the adsDataCustomerId used in the query.
 *
 * <p>Note that this sample is intended to be run with the query created in
 * CreateUserListQuery.java. Some adjustments to the QueryExecutionSpec may be required for this
 * example to start a different query.
 *
 * <p>See the <a href="customers.userListQueries.start reference
 * documentation">https://developers.google.com/ads-data-hub/reference/rest/v1/customers.userListQueries/start</a>
 * for more details.
 */
public class StartUserListQuery extends BaseSample {
  private static final DateTimeFormatter DATE_FORMATTER = DateTimeFormat.forPattern("yyyy-MM-dd");

  @Override
  public String getName() {
    return "Start user list query";
  }

  @Override
  public String getDescription() {
    return "Starts user list query with the specified name.";
  }

  @Override
  public void execute(AdsDataHub client) throws IOException {
    DateTime defaultEndTime = DateTime.now();
    DateTime defaultStartTime = defaultEndTime.minusDays(7);

    Long customerId = getOptionalLongInput("customerId", "Enter the customer ID");
    String name =
        getStringInput(
            "userListQueryName",
            "Enter the fully qualified resource name for the query to be run (Format:"
                + " \"customers/[customer_id]/userListQueries/[resource_id]\")");
    Long userListId =
        getLongInput("userListId", "Enter the id of the user list to add users to");
    Long adsDataCustomerId =
        getOptionalLongInput(
            "adsDataCustomerId",
            "Linked Ads Data Hub customer ID that owns the ads data used in the query");
    Long matchDataCustomerId =
        getOptionalLongInput(
            "matchDataCustomerId",
            "Linked Ads Data Hub customer ID that owns the match data for the query");
    Long campaignId = getLongInput("campaignId", "Campaign id used to filter events in the query");
    String startDate =
        getStringInput(
            "startDate",
            "Enter the start date in YYYY-MM-DD format.",
            DATE_FORMATTER.print(defaultStartTime));
    String endDate =
        getStringInput(
            "endDate",
            "Enter the end date in YYYY-MM-DD format.",
            DATE_FORMATTER.print(defaultEndTime));
    String timeZone = getOptionalStringInput("timeZone", "Enter the time zone");

    DateTime start = DateTime.parse(startDate, DATE_FORMATTER);
    DateTime end = DateTime.parse(endDate, DATE_FORMATTER);

    StartUserListQueryRequest request =
        new StartUserListQueryRequest()
            .setSpec(
                new QueryExecutionSpec()
                    .setStartDate(
                        new Date()
                            .setDay(start.getDayOfMonth())
                            .setMonth(start.getMonthOfYear())
                            .setYear(start.getYear()))
                    .setEndDate(
                        new Date()
                            .setDay(end.getDayOfMonth())
                            .setMonth(end.getMonthOfYear())
                            .setYear(end.getYear()))
                    .setTimeZone(timeZone)
                    .setAdsDataCustomerId(adsDataCustomerId)
                    .setMatchDataCustomerId(matchDataCustomerId)
                    .setParameterValues(
                        ImmutableMap.of(
                            "campaign_id", new ParameterValue().setValue(campaignId.toString()))))
            .setUserListId(userListId)
            .setCustomerId(customerId);

    Operation operation = client.customers().userListQueries().start(name, request).execute();
    Boolean done = operation.getDone();

    System.out.println("========================================");
    System.out.printf("Running user list query with name \"%s\" via the following operation:%n", name);
    System.out.println("========================================");
    System.out.printf("Name: %s\n", operation.getName());
    System.out.printf("\tMetadata: %s\n", operation.getMetadata());
    System.out.printf("\tDone: %s\n", done != null ? done : false);

    if (done != null && done.equals(true)) {
      Status error = operation.getError();
      if (error != null) {
        System.out.printf("\tError: %s\n", error);
      }
    }
  }
}
